// This is an independent project of an individual developer. Dear PVS-Studio, please check it.
// PVS-Studio Static Code Analyzer for C, C++, C#, and Java: http://www.viva64.com
#include <string.h>
#include <errno.h>
#include "nrf.h"

#include "def.h"
#include "xfTimeout.h"
#include "nrf_errno.h"
#include "bsd_os.h"
#include "bsd_platform.h"

FILENUM(94441);

#define BSD_OS_TRACE_IRQ          EGU2_IRQn
#define BSD_OS_TRACE_IRQ_PRIORITY 6

#define BSD_APPLICATION_IRQ       EGU1_IRQn

int32_t bsd_os_timedwait(uint32_t context, int32_t *timeout)
{
    UNUSED(context);
    // We can't do an infinite wait
    if(*timeout < 0)
    {
        return(0);
    }

    Int16 timeoutHandle = xfTimeout_CreateTimeout(*timeout);
    assert(timeoutHandle >= 0);

    while(false == xfTimeout_HasTimedOut(timeoutHandle))
    {
        __asm("nop");
    }
    xfTimeout_ReleaseTimeout(timeoutHandle);

    return(NRF_ETIMEDOUT);

    //return(0);
}



/// Function required by BSD library. We need to set the EGU1 interrupt.
void bsd_os_application_irq_set(void)
{
    NVIC_SetPendingIRQ(BSD_APPLICATION_IRQ);
}

/// Function required by BSD library. We need to clear the EGU1 interrupt.
void bsd_os_application_irq_clear(void)
{
    NVIC_ClearPendingIRQ(BSD_APPLICATION_IRQ);
}

/// Function required by BSD library. We need to set the EGU2 interrupt.
void bsd_os_trace_irq_set(void)
{
    NVIC_SetPendingIRQ(BSD_OS_TRACE_IRQ);
}

/// Function required by BSD library. We need to clear the EGU2 interrupt.
void bsd_os_trace_irq_clear(void)
{
    NVIC_ClearPendingIRQ(BSD_OS_TRACE_IRQ);
}

void read_task_create(void)
{
    // The read task is achieved using SW interrupt.
    NVIC_SetPriority(BSD_APPLICATION_IRQ, BSD_APPLICATION_IRQ_PRIORITY);
    NVIC_ClearPendingIRQ(BSD_APPLICATION_IRQ);
    NVIC_EnableIRQ(BSD_APPLICATION_IRQ);
}

void trace_task_create(void)
{
    NVIC_SetPriority(BSD_OS_TRACE_IRQ, BSD_OS_TRACE_IRQ_PRIORITY);
    NVIC_ClearPendingIRQ(BSD_OS_TRACE_IRQ);
    NVIC_EnableIRQ(BSD_OS_TRACE_IRQ);
}

/// Function required by BSD library.
void bsd_os_init(void)
{
    read_task_create();
    trace_task_create();
}



/// Function required by BSD library
int32_t bsd_os_trace_put(const uint8_t *const data, uint32_t len)
{
    UNUSED(data);
    UNUSED(len);
    return(0);
}


/// Function required by BSD library. Stores an error code we can read later.
void bsd_os_errno_set(int err_code)
{
    // Translate nrf_errno.h errno to the OS specific value.
    errno = err_code;
}


// void IPC_IRQHandler(void)
// {
// }

void EGU1_IRQHandler(void)
{
    bsd_os_application_irq_handler();
}

void EGU2_IRQHandler(void)
{
    bsd_os_trace_irq_handler();
}
